import json
import geopandas as gpd
from shapely.geometry import Polygon


def get_relation(self, body_of_water: str):
    # Load GeoJSON data using geopandas
    geo_data = gpd.read_file("server/map/mjosa.geojson")

    # Filter only polygons, exclude points and other feature types
    polygon_data = geo_data[geo_data['geometry'].geom_type == 'Polygon']

    # Extract coordinates from polygons and create polygon objects
    polygons = [Polygon(polygon.exterior) for polygon in polygon_data['geometry']]

    if len(polygons) <= 1:
        print("Failed to convert to polygons")

    # Convert response data to JSON string
    response_json = json.dumps(divide_relation(polygons))

    # Set headers
    self.send_response(200)
    self.send_header("Content-type", "application/json")
    self.end_headers()

    # Write coordinates to response object
    self.wfile.write(response_json.encode('utf-8'))


def divide_relation(polygons):
    # Define tile size
    tile_size = 0.01
    id = 1
    tiles = []

    for polygon in polygons:
        x_min, y_min, x_max, y_max = polygon.bounds
        rows = int((y_max - y_min) / tile_size)
        cols = int((x_max - x_min) / tile_size)

        if rows == 0 or cols == 0:  # Skip small polygons
            continue

        for row in range(rows):
            for col in range(cols):
                tile_bbox = Polygon([
                    (x_min + col * tile_size, y_min + row * tile_size),
                    (x_min + (col + 1) * tile_size, y_min + row * tile_size),
                    (x_min + (col + 1) * tile_size, y_min + (row + 1) * tile_size),
                    (x_min + col * tile_size, y_min + (row + 1) * tile_size)
                ])
                tiles.append({"SubDivID": id, "polygon": tile_bbox})
                id += 1

    if len(tiles) <= 1:
        print("Failed to divide polygons into tiles")

    tiles_json = [{"SubDivID": tile["SubDivID"], "coordinates": list(tile["polygon"].exterior.coords)} for tile in
                  tiles]
    return tiles_json
